# -*- coding: utf-8 -*-
import os
import shutil
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib


class TestCase(systemtools.TestCase):

    test_path = pathlib.Path(__file__).parent
    doctest_path = test_path.joinpath('doctest_test')

    failing_module_path = doctest_path.joinpath('doctest_fail.py')
    failing_module_contents = stringtools.normalize(r'''
        def fail():
            """
            A failing module.

            ::

                >>> True is False
                True

            """
            return None
    ''')

    passing_module_path = doctest_path.joinpath('doctest_pass.py')
    passing_module_contents = stringtools.normalize(r'''
        def pass():
            """
            A passing module.

            ::

                >>> True is False
                False

            """
            return None
    ''')

    def setUp(self):
        super(TestCase, self).setUp()
        if not self.doctest_path.exists():
            self.doctest_path.mkdir()
        with open(str(self.failing_module_path), 'w') as file_pointer:
            file_pointer.write(self.failing_module_contents)
        with open(str(self.passing_module_path), 'w') as file_pointer:
            file_pointer.write(self.passing_module_contents)

    def tearDown(self):
        super(TestCase, self).tearDown()
        shutil.rmtree(str(self.doctest_path))

    def test_both(self):
        script = commandlinetools.DoctestScript()
        command = [str(self.doctest_path)]
        with systemtools.TemporaryDirectoryChange(str(self.test_path)):
            with systemtools.RedirectedStreams(stdout=self.string_io):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
        assert context_manager.exception.code == 1
        script_output = self.ansi_escape.sub('', self.string_io.getvalue())
        script_output = stringtools.normalize(script_output)
        expected = stringtools.normalize('''
        doctest_test/doctest_fail.py FAILED
        doctest_test/doctest_pass.py OK

        **********************************************************************
        File ".../doctest_test/doctest_fail.py", line 7, in doctest_fail.py
        Failed example:
            True is False
        Expected:
            True
        Got:
            False
        **********************************************************************
        1 items had failures:
            1 of   1 in doctest_fail.py
        ***Test Failed*** 1 failures.

        FAILED: doctest_test/doctest_fail.py

        1 passed, 1 failed out of 2 tests in 2 modules.
        '''.replace('/', os.path.sep))
        self.compare_strings(expected, script_output)

    def test_diff(self):
        script = commandlinetools.DoctestScript()
        command = ['--diff', str(self.failing_module_path)]
        with systemtools.TemporaryDirectoryChange(str(self.test_path)):
            with systemtools.RedirectedStreams(stdout=self.string_io):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
        assert context_manager.exception.code == 1
        script_output = self.ansi_escape.sub('', self.string_io.getvalue())
        script_output = stringtools.normalize(script_output)
        expected = stringtools.normalize('''
        doctest_test/doctest_fail.py FAILED

        **********************************************************************
        File ".../doctest_test/doctest_fail.py", line 7, in doctest_fail.py
        Failed example:
            True is False
        Differences (ndiff with -expected +actual):
            - True
            + False
        **********************************************************************
        1 items had failures:
            1 of   1 in doctest_fail.py
        ***Test Failed*** 1 failures.

        FAILED: doctest_test/doctest_fail.py

        0 passed, 1 failed out of 1 test in 1 module.
        '''.replace('/', os.path.sep))
        self.compare_strings(expected, script_output)

    def test_fail(self):
        script = commandlinetools.DoctestScript()
        command = [str(self.failing_module_path)]
        with systemtools.TemporaryDirectoryChange(str(self.test_path)):
            with systemtools.RedirectedStreams(stdout=self.string_io):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
        assert context_manager.exception.code == 1
        script_output = self.ansi_escape.sub('', self.string_io.getvalue())
        script_output = stringtools.normalize(script_output)
        expected = stringtools.normalize('''
        doctest_test/doctest_fail.py FAILED

        **********************************************************************
        File ".../doctest_test/doctest_fail.py", line 7, in doctest_fail.py
        Failed example:
            True is False
        Expected:
            True
        Got:
            False
        **********************************************************************
        1 items had failures:
            1 of   1 in doctest_fail.py
        ***Test Failed*** 1 failures.

        FAILED: doctest_test/doctest_fail.py

        0 passed, 1 failed out of 1 test in 1 module.
        '''.replace('/', os.path.sep))
        self.compare_strings(expected, script_output)

    def test_pass(self):
        script = commandlinetools.DoctestScript()
        command = [str(self.passing_module_path)]
        with systemtools.TemporaryDirectoryChange(str(self.test_path)):
            with systemtools.RedirectedStreams(stdout=self.string_io):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
        assert context_manager.exception.code == 0
        script_output = self.ansi_escape.sub('', self.string_io.getvalue())
        script_output = stringtools.normalize(script_output)
        expected = stringtools.normalize('''
        doctest_test/doctest_pass.py OK

        1 passed, 0 failed out of 1 test in 1 module.
        '''.replace('/', os.path.sep))
        self.compare_strings(expected, script_output)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class StaffChange(AbjadValueObject):
    r'''Staff change.

    ..  container:: example

        **Example 1.** Explicit staff change:

        ::

            >>> staff_group = StaffGroup()
            >>> staff_group.context_name = 'PianoStaff'
            >>> rh_staff = Staff("c'8 d'8 e'8 f'8")
            >>> rh_staff.name = 'RHStaff'
            >>> lh_staff = Staff("s2")
            >>> lh_staff.name = 'LHStaff'
            >>> staff_group.extend([rh_staff, lh_staff])
            >>> staff_change = indicatortools.StaffChange(lh_staff)
            >>> attach(staff_change, rh_staff[2])
            >>> show(staff_group) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff_group))
            \new PianoStaff <<
                \context Staff = "RHStaff" {
                    c'8
                    d'8
                    \change Staff = LHStaff
                    e'8
                    f'8
                }
                \context Staff = "LHStaff" {
                    s2
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_staff',
        )

    _format_leaf_children = False

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, staff=None):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        if not isinstance(staff, (scoretools.Staff, type(None))):
            message = 'staff change input value {!r}'
            message += ' must be staff instance.'
            message.format(staff)
            raise TypeError(message)
        self._staff = staff

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of staff change.

        ..  container:: example

            **Example 1.** Default staff change:

            ::

                >>> staff_change = indicatortools.StaffChange()
                >>> print(str(staff_change))
                \change Staff = ##f

        ..  container:: example

            **Example 2.** Explicit staff change:
    
            ::

                >>> staff_change = indicatortools.StaffChange(staff=lh_staff)
                >>> print(str(staff_change))
                \change Staff = LHStaff

        Returns string.
        '''
        from abjad.tools import schemetools
        if self.staff is None:
            return r'\change Staff = ##f'
        return r'\change Staff = {}'.format(
            schemetools.Scheme.format_scheme_value(self.staff.name),
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.staff)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.opening.commands.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of staff change.

        ..  container:: example

            **Example 1.** Default staff change:

            ::

                >>> staff_change = indicatortools.StaffChange()
                >>> staff_change.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        ..  container:: example

            **Example 2.** Explicit staff change:
    
            ::

                >>> staff_change = indicatortools.StaffChange(staff=lh_staff)
                >>> staff_change.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Staff changes are staff-scoped.

        Returns staff.
        '''
        return self._default_scope

    @property
    def staff(self):
        r'''Gets staff of staff change.

        ..  container:: example

            **Example 1.** Default staff change:

            ::

                >>> staff_change = indicatortools.StaffChange()
                >>> staff_change.staff is None
                True

        ..  container:: example

            **Example 2.** Explicit staff change:
    
            ::

                >>> staff_change = indicatortools.StaffChange(staff=lh_staff)
                >>> staff_change.staff
                Staff('s2')
        
        Set to staff or none.

        Defaults to none.

        Returns staff or none.
        '''
        return self._staff



# -*- coding: utf-8 -*-
import six


def capitalize_start(string):
    r'''Capitalizes `string`.

    ..  container:: example

        ::

            >>> string = 'violin I'

    ..  container:: example

        ::

            >>> stringtools.capitalize_start(string)
            'Violin I'

    Function differs from built-in ``string.capitalize()``.

    This function affects only ``string[0]`` and leaves noninitial characters
    as-is.

    Built-in ``string.capitalize()`` forces noninitial characters to lowercase.

    ..  container:: example

        ::

            >>> string.capitalize()
            'Violin i'

    Returns newly constructed string.
    '''
    if not isinstance(string, six.string_types):
        raise ValueError(repr(string))
    if not string:
        return string
    else:
        return string[0].upper() + string[1:]


# -*- coding: utf-8 -*-


def partition_integer_into_units(n):
    r'''Partitions integer `n` into units.

    ..  container:: example

        With positive integer:

        ::

            >>> mathtools.partition_integer_into_units(6)
            [1, 1, 1, 1, 1, 1]

    ..  container:: example

        With negative integer:

        ::

            >>> mathtools.partition_integer_into_units(-5)
            [-1, -1, -1, -1, -1]

    ..  container:: example

        With zero:

        ::

            >>> mathtools.partition_integer_into_units(0)
            []

    Returns list of zero or more parts with absolute value equal to 1.
    '''
    from abjad.tools import mathtools
    result = abs(n) * [mathtools.sign(n) * 1]
    return result



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment___hash___01():
    r'''Vertical moments behave well when included in a set.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    vms = []
    vms.extend(iterate(staff).by_vertical_moment())
    vms.extend(iterate(staff).by_vertical_moment())

    assert len(vms) == 8
    assert len(set(vms)) == 4



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_copy_01():
    r'''Deep copies components.
    Deep copies spanners that attach to client.
    Fractures spanners that attach to components not in client.
    Returns Python list of copied components.
    '''

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, leaves)
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ ( \startTrillSpan
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ] ) \stopTrillSpan
            }
        }
        '''
        )

    selection = select(leaves[2:4])
    result = mutate(selection).copy()
    new = Voice(result)

    assert format(new) == stringtools.normalize(
        r'''
        \new Voice {
            e'8 [ ( \startTrillSpan
            f'8 ] ) \stopTrillSpan
        }
        '''
        )
    assert inspect_(voice).is_well_formed()
    assert inspect_(new).is_well_formed()


def test_agenttools_MutationAgent_copy_02():
    r'''Copy one measure and fracture spanners.
    '''

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, leaves)
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ ( \startTrillSpan
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ] ) \stopTrillSpan
            }
        }
        '''
        )

    result = mutate(voice[1:2]).copy()
    new = Voice(result)

    assert format(new) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                e'8 [ ( \startTrillSpan
                f'8 ] ) \stopTrillSpan
            }
        }
        '''
        )
    assert inspect_(voice).is_well_formed()
    assert inspect_(new).is_well_formed()


def test_agenttools_MutationAgent_copy_03():
    r'''Three notes crossing measure boundaries.
    '''

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, leaves)
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ ( \startTrillSpan
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ] ) \stopTrillSpan
            }
        }
        '''
        )

    selection = select(leaves[-3:])
    result = mutate(selection).copy()
    new = Voice(result)

    assert format(new) == stringtools.normalize(
        r'''
        \new Voice {
            f'8 [ ( \startTrillSpan
            g'8
            a'8 ] ) \stopTrillSpan
        }
        '''
        )
    assert inspect_(voice).is_well_formed()
    assert inspect_(new).is_well_formed()


def test_agenttools_MutationAgent_copy_04():
    r'''Optional 'n' argument for multiple copies.
    '''

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, leaves)
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ ( \startTrillSpan
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ] ) \stopTrillSpan
            }
        }
        '''
        )

    result = mutate(voice[1:2]).copy(n=3)
    new = Voice(result)

    assert format(new) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                e'8 [ ( \startTrillSpan
                f'8 ] ) \stopTrillSpan
            }
            {
                e'8 [ ( \startTrillSpan
                f'8 ] ) \stopTrillSpan
            }
            {
                e'8 [ ( \startTrillSpan
                f'8 ] ) \stopTrillSpan
            }
        }
        '''
        )
    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_copy_05():

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ (
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8 ] )
            }
        }
        '''
        )

    selection = selectiontools.Selection(music=voice)
    new_selection = mutate(selection).copy()
    new_voice = new_selection[0]
    for component in iterate(new_voice).by_class():
        detach(spannertools.Spanner, component)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_06():

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ (
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8 ] )
            }
        }
        '''
        )

    result = mutate(voice[1:]).copy()
    new_voice = Voice(result)
    for component in iterate(new_voice).by_class():
        detach(spannertools.Spanner, component)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )
    assert inspect_(voice).is_well_formed()
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_07():

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ (
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8 ] )
            }
        }
        '''
        )

    leaves = select(leaves[:6])
    result = mutate(leaves).copy()
    new_voice = Voice(result)
    for component in iterate(new_voice).by_class():
        detach(spannertools.Spanner, component)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
        '''
        )
    assert inspect_(voice).is_well_formed()
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_08():

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ (
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8 ] )
            }
        }
        '''
        )

    result = mutate(voice[-2:]).copy()
    new_voice = Voice(result)
    for component in iterate(new_voice).by_class():
        detach(spannertools.Spanner, component)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_09():

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8 [ (
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8 ] )
            }
        }
        '''
        )

    result = mutate(voice[-2:]).copy(n=3)
    new_voice = Voice(result)
    for component in iterate(new_voice).by_class():
        detach(spannertools.Spanner, component)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_10():
    r'''Copies hairpin.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    crescendo = Crescendo()
    attach(crescendo, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    new_notes = mutate(staff[:4]).copy()
    staff.extend(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            e'8
            f'8
            fs'8
            g'8
            c'8 \<
            cs'8
            d'8
            ef'8 \!
        }
        '''
        )
    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_copy_11():
    r'''Copy consecutive notes across tuplet boundary in staff.
    Includes enclosing containers.
    '''

    staff = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 g'8 a'8 }")
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    leaves = select(leaves[1:5])
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'8
                e'8
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                f'8
                g'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_12():
    r'''Copy consecutive notes across tuplet boundary in voice and staff.
    Includes enclosing containers.
    '''

    voice = Voice(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 g'8 a'8 }")
    staff = Staff([voice])
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                \times 2/3 {
                    c'8
                    d'8
                    e'8
                }
                \times 2/3 {
                    f'8
                    g'8
                    a'8
                }
            }
        }
        '''
        )

    leaves = select(leaves[1:5])
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                \tweak edge-height #'(0.7 . 0)
                \times 2/3 {
                    d'8
                    e'8
                }
                \tweak edge-height #'(0.7 . 0)
                \times 2/3 {
                    f'8
                    g'8
                }
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_13():
    r'''Works fine on voices nested inside simultaneous context.
    Includes enclosing containers.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("g'8 a'8 b'8 c''8")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff <<
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        >>
        '''
        )

    leaves = select(leaves[1:3])
    new_voice = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            d'8
            e'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_14():
    r'''Copy consecutive notes in measure with power-of-two denominator.
    Includes enclosing containers.
    '''

    measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
    leaves = measure[1:3]
    new_measure = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            d'8
            e'8
        }
        '''
        )

    assert inspect_(new_measure).is_well_formed()


def test_agenttools_MutationAgent_copy_15():
    r'''Copy consecutive notes in staff and score.
    Includes enclosing containers.
    '''

    score = Score([Staff("c'8 d'8 e'8 f'8")])
    staff = score[0]
    leaves = staff[1:3]
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            d'8
            e'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_16():
    r'''Copy consecutive leaves from tuplet in measure with power-of-two
    denominator. Measure without power-of-two denominator results.
    Includes enclosing containers.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(4, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8")
    measure = Measure((4, 8), [tuplet])
    measure.implicit_scaling = True

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 4/8
            \times 4/5 {
                c'8
                d'8
                e'8
                f'8
                g'8
            }
        }
        '''
        )

    leaves = select(measure).by_leaf()
    leaves = select(leaves[1:4])
    new_measure = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_measure) == stringtools.normalize(
        r'''
        {
            \time 3/10
            \scaleDurations #'(4 . 5) {
                {
                    d'8
                    e'8
                    f'8
                }
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()
    assert inspect_(new_measure).is_well_formed()


def test_agenttools_MutationAgent_copy_17():
    r'''Copy consecutive leaves from tuplet in measure and voice.
    Measure without power-of-two time signature denominator results.
    Includes enclosing containers.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(4, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8")
    measure = Measure((4, 8), [tuplet])
    measure.implicit_scaling = True
    voice = Voice([measure])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 4/8
                \times 4/5 {
                    c'8
                    d'8
                    e'8
                    f'8
                    g'8
                }
            }
        }
        '''
        )

    leaves = select(voice).by_leaf()
    leaves = select(leaves[1:4])
    new_voice = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/10
                \scaleDurations #'(4 . 5) {
                    {
                        d'8
                        e'8
                        f'8
                    }
                }
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert inspect_(new_voice).is_well_formed()


def test_agenttools_MutationAgent_copy_18():
    r'''Measures shrink when copying a partial tuplet.
    Note that test only works with fixed-duration tuplets.
    Includes enclosing containers.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet((2, 8), "c'8 d'8 e'8")
    tuplet_2 = scoretools.FixedDurationTuplet((2, 8), "f'8 g'8 a'8")
    measure = Measure((4, 8), [tuplet_1, tuplet_2])
    measure.implicit_scaling = True
    leaves = select(measure).by_leaf()

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 4/8
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    leaves = select(leaves[1:])
    new_measure = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_measure) == stringtools.normalize(
        r'''
        {
            \time 5/12
            \scaleDurations #'(2 . 3) {
                {
                    d'8
                    e'8
                }
                {
                    f'8
                    g'8
                    a'8
                }
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()
    assert inspect_(new_measure).is_well_formed()


def test_agenttools_MutationAgent_copy_19():
    r'''Copy consecutive leaves across measure boundary.
    Includes enclosing containers.
    '''

    measure_1 = Measure((3, 8), "c'8 d'8 e'8")
    measure_2 = Measure((3, 8), "f'8 g'8 a'8")
    staff = Staff([measure_1, measure_2])
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/8
                c'8
                d'8
                e'8
            }
            {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    leaves = select(leaves[2:4])
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                e'8
            }
            {
                f'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_20():
    r'''Copy consecutive leaves from tuplet in staff;
    pass start and stop indices local to tuplet.
    Includes enclosing containers.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet((2, 8), "c'8 d'8 e'8")
    tuplet_2 = scoretools.FixedDurationTuplet((2, 8), "f'8 g'8 a'8")
    staff = Staff([tuplet_1, tuplet_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    leaves = tuplet_2[1:3]
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_21():
    r'''Copy consecutive leaves from measure in staff;
    pass start and stop indices local to measure.
    Includes enclosing containers.
    '''

    measure_1 = Measure((3, 8), "c'8 d'8 e'8")
    measure_2 = Measure((3, 8), "f'8 g'8 a'8")
    staff = Staff([measure_1, measure_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/8
                c'8
                d'8
                e'8
            }
            {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    leaves = measure_2[1:3]
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_22():
    r'''Copy consecutive leaves from in-staff measure without
    power-of-two denominator. Pass start and stop indices local to measure.
    Includes enclosing containers.
    '''

    measure_1 = Measure((3, 9), "c'8 d'8 e'8")
    measure_1.implicit_scaling = True
    measure_2 = Measure((3, 9), "f'8 g'8 a'8")
    measure_2.implicit_scaling = True
    staff = Staff([measure_1, measure_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/9
                \scaleDurations #'(8 . 9) {
                    c'8
                    d'8
                    e'8
                }
            }
            {
                \scaleDurations #'(8 . 9) {
                    f'8
                    g'8
                    a'8
                }
            }
        }
        '''
        )

    leaves = measure_2[1:3]
    new_staff = mutate(leaves).copy(include_enclosing_containers=True)

    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/9
                \scaleDurations #'(8 . 9) {
                    g'8
                    a'8
                }
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert inspect_(new_staff).is_well_formed()


def test_agenttools_MutationAgent_copy_23():
    r'''Copy indicators while maintaining "is_annotation" flag.
    '''

    old_staff = Staff("c'4 d'4 e'4 f'4")
    indicator = indicatortools.BendAfter()
    attach(indicator, old_staff[0], is_annotation=True)
    assert format(old_staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }
        ''')
    assert inspect_(old_staff[0]).get_indicators() == (indicator,)

    new_staff = mutate(old_staff).copy()
    assert format(new_staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }
        ''')
    assert inspect_(new_staff[0]).get_indicators() == (indicator,)

    assert inspect_(old_staff[0]).get_indicators()[0] is not \
        inspect_(new_staff[0]).get_indicators()[0]



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_notes_01():
    r'''Can take a single pitch and a single duration.
    '''

    note = scoretools.make_notes(1, (1,4))
    assert isinstance(note, selectiontools.Selection)
    assert len(note) == 1
    assert isinstance(note[0], Note)
    assert note[0].written_duration == Duration(1, 4)
    assert all(len(inspect_(x).get_logical_tie()) == 1 for x in note)


def test_scoretools_make_notes_02():
    r'''Tied durations result in more than one tied note.
    '''

    note = scoretools.make_notes(1, (5, 8))
    assert len(note) == 2
    assert isinstance(note[0], Note)
    assert isinstance(note[1], Note)
    assert note[0].written_duration == Duration(4, 8)
    assert note[1].written_duration == Duration(1, 8)
    assert all(len(inspect_(x).get_logical_tie()) == 2 for x in note)


def test_scoretools_make_notes_03():
    r'''May take a list of pitches and a single duration.
    '''

    t = scoretools.make_notes([1, 2], (1, 4))
    assert len(t) == 2
    assert t[0].written_pitch.numbered_pitch == 1
    assert t[1].written_pitch.numbered_pitch == 2


def test_scoretools_make_notes_04():
    r'''May take a single pitch and a list of duration.
    '''

    may = scoretools.make_notes(1, [(1, 8), (1, 4)])
    assert len(may) == 2
    assert may[0].written_pitch.numbered_pitch == 1
    assert may[1].written_pitch.numbered_pitch == 1
    assert may[0].written_duration == Duration(1, 8)
    assert may[1].written_duration == Duration(1, 4)


def test_scoretools_make_notes_05():
    r'''May take a list of pitches and list of durations.
    '''

    duration = scoretools.make_notes([0, 1], [(1, 8), (1, 4)])
    assert len(duration) == 2
    assert duration[0].written_pitch.numbered_pitch == 0
    assert duration[1].written_pitch.numbered_pitch == 1
    assert duration[0].written_duration == Duration(1, 8)
    assert duration[1].written_duration == Duration(1, 4)


def test_scoretools_make_notes_06():
    r'''Durations can be durations.
    '''

    duration = scoretools.make_notes(1, Duration(1, 4))
    assert len(duration) == 1
    assert duration[0].written_duration == Duration(1, 4)


def test_scoretools_make_notes_07():
    r'''Durations can be a list of durations.
    '''

    duration = scoretools.make_notes(1, [Duration(1, 4)])
    assert len(duration) == 1
    assert duration[0].written_duration == Duration(1, 4)


def test_scoretools_make_notes_08():
    r'''Decrease durations monotonically.
    '''

    true = scoretools.make_notes(1, (5, 16),
        decrease_durations_monotonically=True)
    assert len(true) == 2
    assert true[0].written_duration == Duration(4, 16)
    assert true[1].written_duration == Duration(1, 16)


def test_scoretools_make_notes_09():
    r'''Increase durations monotonically.
    '''

    false = scoretools.make_notes(1, (5, 16),
        decrease_durations_monotonically=False)
    assert len(false) == 2
    assert false[0].written_duration == Duration(1, 16)
    assert false[1].written_duration == Duration(4, 16)


def test_scoretools_make_notes_10():
    r'''Make non-power-of-two duration.
    '''

    tuplet = scoretools.make_notes(1, (1, 36))
    assert len(tuplet) == 1
    assert isinstance(tuplet[0], Tuplet)
    assert len(tuplet[0]) == 1
    assert inspect_(tuplet[0]).get_duration() == Duration(1, 36)
    assert inspect_(tuplet[0][0]).get_duration() == Duration(1, 36)
    assert tuplet[0][0].written_duration == Duration(1, 32)


def test_scoretools_make_notes_11():
    r'''Make multiple non-power-or-two durations.
    '''

    tuplet = scoretools.make_notes(1, [(1, 12), (1, 6), (1, 8)])

    r'''
    \times 2/3 {
        cs'8
        cs'4
    }
    cs'8
    '''

    assert len(tuplet) == 2
    assert isinstance(tuplet[0], Tuplet)
    assert isinstance(tuplet[1], Note)
    assert len(tuplet[0]) == 2
    assert inspect_(tuplet[0]).get_duration() == Duration(3, 12)
    assert inspect_(tuplet[0][0]).get_duration() == Duration(1, 12)
    assert inspect_(tuplet[0][1]).get_duration() == Duration(1, 6)
    assert tuplet[0][0].written_duration == Duration(1, 8)
    assert tuplet[0][1].written_duration == Duration(1, 4)
    assert tuplet[1].written_duration == Duration(1, 8)


def test_scoretools_make_notes_12():

    t = scoretools.make_notes(1, [(5, 12), (1, 6), (1, 8)],
        decrease_durations_monotonically=False)
    assert len(t) == 2
    assert isinstance(t[0], Tuplet)
    assert isinstance(t[1], Note)
    assert len(t[0]) == 3
    assert inspect_(t[0]).get_duration() == Duration(7, 12)
    assert inspect_(t[0][0]).get_duration() == Duration(1, 12)
    assert inspect_(t[0][1]).get_duration() == Duration(4, 12)
    assert inspect_(t[0][2]).get_duration() == Duration(1, 6)
    assert t[0][0].written_duration == Duration(1, 8)
    assert t[0][1].written_duration == Duration(4, 8)
    assert t[0][2].written_duration == Duration(1, 4)
    assert t[1].written_duration == Duration(1, 8)



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import override


def make_piano_sketch_score_from_leaves(leaves, lowest_treble_pitch=None):
    r'''Make piano sketch score from `leaves`:

    ::

        >>> notes = scoretools.make_notes(
        ...     [-12, -10, -8, -7, -5, 0, 2, 4, 5, 7],
        ...     [(1, 16)],
        ...     )
        >>> score, treble_staff, bass_staff = \
        ...     scoretools.make_piano_sketch_score_from_leaves(notes)

    ..  doctest::

        >>> print(format(score))
        \new Score \with {
            \override BarLine.stencil = ##f
            \override BarNumber.transparent = ##t
            \override SpanBar.stencil = ##f
            \override TimeSignature.stencil = ##f
        } <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    r16
                    r16
                    r16
                    r16
                    r16
                    c'16
                    d'16
                    e'16
                    f'16
                    g'16
                }
                \context Staff = "bass" {
                    \clef "bass"
                    c16
                    d16
                    e16
                    f16
                    g16
                    r16
                    r16
                    r16
                    r16
                    r16
                }
            >>
        >>

    ::

        >>> show(score) # doctest: +SKIP

    When ``lowest_treble_pitch=None`` set to B3.

    Make time signatures and bar numbers transparent.

    Do not print bar lines or span bars.

    Returns score, treble staff, bass staff.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import lilypondfiletools
    from abjad.tools import markuptools
    from abjad.tools import pitchtools
    from abjad.tools import scoretools

    if lowest_treble_pitch is None:
        lowest_treble_pitch = pitchtools.NamedPitch('b')

    # make and configure score
    score, treble_staff, bass_staff = \
        scoretools.make_piano_score_from_leaves(leaves, lowest_treble_pitch)
    override(score).time_signature.stencil = False
    override(score).bar_number.transparent = True
    override(score).bar_line.stencil = False
    override(score).span_bar.stencil = False

    # make and configure lily file
    lilypond_file = lilypondfiletools.LilyPondFile.new(score)
    lilypond_file.layout_block.indent = 0
    lilypond_file.paper_block.tagline = markuptools.Markup('')

    # return score, treble staff, bass staff
    return score, treble_staff, bass_staff



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


# TODO: dynamics should accept direction strings.
def test_lilypondparsertools_LilyPondParser__indicators__Dynamic_01():

    target = Staff(Note(-12, (1, 2)) * 6)
    dynamic = Dynamic('ppp')
    attach(dynamic, target[0])
    dynamic = Dynamic('mp')
    attach(dynamic, target[1])
    dynamic = Dynamic('rfz')
    attach(dynamic, target[2])
    dynamic = Dynamic('mf')
    attach(dynamic, target[3])
    dynamic = Dynamic('spp')
    attach(dynamic, target[4])
    dynamic = Dynamic('ff')
    attach(dynamic, target[5])

    string = r'''\new Staff { c2\ppp c\mp c2\rfz c\mf c2\spp c\ff }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result
    for x in result:
        dynamics = inspect_(x).get_indicators(Dynamic)
        assert len(dynamics) == 1



# -*- coding: utf-8 -*-
from abjad import *


def test_durationtools_Duration___mul___01():
    r'''Multiplying duration by nonreduced fraction gives nonreduced fraction.
    '''

    result = Duration(1) * mathtools.NonreducedFraction(3, 3)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (3, 3)

    result = mathtools.NonreducedFraction(3, 3) * Duration(1)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (3, 3)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeNode_proper_parentage_01():

    leaf = datastructuretools.TreeNode()
    subsubcontainer = datastructuretools.TreeContainer()
    subcontainer = datastructuretools.TreeContainer()
    container = datastructuretools.TreeContainer()

    container.append(subcontainer)
    subcontainer.append(subsubcontainer)
    subsubcontainer.append(leaf)

    assert leaf.proper_parentage == (subsubcontainer, subcontainer, container)
    assert subsubcontainer.proper_parentage == (subcontainer, container)
    assert subcontainer.proper_parentage == (container,)
    assert container.proper_parentage == ()



# -*- coding: utf-8 -*-
from abjad.tools.schemetools import Scheme
from abjad.tools.schemetools import SchemePair


def test_schemetools_Scheme_01():
    r'''Scheme can be initialized from any value.
    '''

    scheme = Scheme(True)
    scheme = Scheme(False)
    scheme = Scheme(None)
    scheme = Scheme('hello')
    scheme = Scheme('hello world')
    scheme = Scheme((Scheme('foo'), Scheme(3.14159)))
    scheme = Scheme((SchemePair('padding', 1), SchemePair('attach-dir', -1)))


def test_schemetools_Scheme_02():
    r'''Scheme takes an optional `quoting` keyword, for prepending
    quote/unquote ticks.
    '''

    scheme = Scheme(('fus', 'ro', 'dah'), quoting = "',")
    assert str(scheme) == "',(fus ro dah)"


def test_schemetools_Scheme_03():
    r'''__str__ of Scheme returns the Scheme formatted value without the hash
    mark, while format(Scheme) returns the formatted value with the hash mark,
    allowing for nested Scheme expressions.
    '''

    scheme = Scheme(('fus', 'ro', 'dah'), quoting = "'")
    assert str(scheme) == "'(fus ro dah)"
    assert format(scheme) == "#'(fus ro dah)"


def test_schemetools_Scheme_04():
    r'''Scheme attempts to format Python values into Scheme equivalents.
    '''

    assert format(Scheme(True)) == '##t'
    assert format(Scheme(False)) == '##f'
    assert format(Scheme(None)) == '##f'
    assert format(Scheme('hello world')) == '#"hello world"'
    assert format(Scheme([1, 2, 3])) == '#(1 2 3)'
    assert format(Scheme((SchemePair('padding', 1), SchemePair('attach-dir', -1)), quoting="'")) == \
        "#'((padding . 1) (attach-dir . -1))"


def test_schemetools_Scheme_05():
    r'''Scheme wraps variable-length arguments into a tuple.
    '''

    assert format(Scheme(1, 2, 3)) == format(Scheme((1, 2, 3)))



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools.datastructuretools.TypedCounter import TypedCounter


class OffsetCounter(TypedCounter):
    r'''An offset counter.

    ..  container:: example

        ::

            >>> timespans = timespantools.TimespanList([
            ...     timespantools.Timespan(0, 16),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8),
            ...     ])
            >>> timespan_operand = timespantools.Timespan(6, 10)
            >>> timespans = timespans - timespan_operand
            >>> offset_counter = metertools.OffsetCounter(timespans)

        ::

            >>> f(offset_counter)
            metertools.OffsetCounter(
                {
                    durationtools.Offset(-2, 1): 1,
                    durationtools.Offset(0, 1): 1,
                    durationtools.Offset(5, 1): 1,
                    durationtools.Offset(6, 1): 3,
                    durationtools.Offset(10, 1): 2,
                    durationtools.Offset(12, 1): 1,
                    durationtools.Offset(16, 1): 1,
                    }
                )

        ::

            >>> show(offset_counter, scale=0.5) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None):
        TypedCounter.__init__(self, item_class=durationtools.Offset)
        if items:
            for item in items:
                if hasattr(item, 'start_offset') and \
                    hasattr(item, 'stop_offset'):
                    self[item.start_offset] += 1
                    self[item.stop_offset] += 1
                elif hasattr(item, '_get_timespan'):
                    self[item._get_timespan().start_offset] += 1
                    self[item._get_timespan().stop_offset] += 1
                else:
                    offset = durationtools.Offset(item)
                    self[offset] += 1

    ### SPECIAL METHODS ###

    def __illustrate__(self, range_=None, scale=None):
        r'''Illustrates offset counter.

        ..  container:: example

            ::

                >>> timespans = timespantools.TimespanList([
                ...     timespantools.Timespan(0, 16),
                ...     timespantools.Timespan(5, 12),
                ...     timespantools.Timespan(-2, 8),
                ...     ])
                >>> timespan_operand = timespantools.Timespan(6, 10)
                >>> timespans = timespans - timespan_operand
                >>> offset_counter = metertools.OffsetCounter(timespans)
                >>> show(offset_counter, scale=0.5) # doctest: +SKIP

        Returns LilyPond file.
        '''
        import abjad
        if not self:
            return markuptools.Markup.null().__illustrate__()
        if isinstance(range_, abjad.Timespan):
            minimum, maximum = range_.start_offset, range_.stop_offset
        elif range_ is not None:
            minimum, maximum = range_
        else:
            minimum, maximum = min(self), max(self)
        minimum = float(durationtools.Offset(minimum))
        maximum = float(durationtools.Offset(maximum))
        if scale is None:
            scale = 1.
        assert 0 < scale
        postscript_scale = 150. / (maximum - minimum)
        postscript_scale *= float(scale)
        postscript_x_offset = (minimum * postscript_scale) - 1
        ps = markuptools.Postscript()
        ps = ps.setlinewidth(0.2)
        ps = ps.setdash([2, 1])
        for offset, count in sorted(self.items()):
            offset = (float(offset) * postscript_scale)
            offset -= postscript_x_offset
            ps = ps.moveto(offset, -1)
            ps = ps.rlineto(0, (float(count) * -3) + 1)
            ps = ps.stroke()
        markup = markuptools.Markup.postscript(ps)
        pieces = [markup]
        for offset in sorted(self):
            offset = durationtools.Multiplier(offset)
            numerator, denominator = offset.numerator, offset.denominator
            fraction = markuptools.Markup.fraction(numerator, denominator)
            fraction = fraction.center_align().fontsize(-3).sans()
            x_translation = (float(offset) * postscript_scale)
            x_translation -= postscript_x_offset
            fraction = fraction.translate((x_translation, 1))
            pieces.append(fraction)
        markup = markuptools.Markup.overlay(pieces)
        return markup.__illustrate__()

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import durationtools
        return durationtools.Offset



# -*- coding: utf-8 -*-
import copy
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class RegistrationComponent(AbjadValueObject):
    '''Registration component.

    ..  container:: example

        Initializes a registration component that specifies that all pitches
        from A0 up to and including C8 should be transposed to the octave
        starting at Eb5 (numbered pitch 15):

        ::

            >>> component = pitchtools.RegistrationComponent('[A0, C8]', 15)
            >>> component
            RegistrationComponent(source_pitch_range=PitchRange(range_string='[A0, C8]'), target_octave_start_pitch=NumberedPitch(15))

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        source_pitch_range='[A0, C8]', 
        target_octave_start_pitch=0,
        ):
        from abjad.tools import pitchtools
        if isinstance(source_pitch_range, pitchtools.PitchRange):
            source_pitch_range = copy.copy(source_pitch_range)
        else:
            source_pitch_range = pitchtools.PitchRange(source_pitch_range)
        target_octave_start_pitch = pitchtools.NumberedPitch(
            target_octave_start_pitch)
        self._source_pitch_range = source_pitch_range
        self._target_octave_start_pitch = target_octave_start_pitch

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a registration component
        with source pitch range and target octave start pitch equal to those of
        this registration component. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.source_pitch_range == argument.source_pitch_range:
                if (self.target_octave_start_pitch ==
                    argument.target_octave_start_pitch):
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats registration component.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __hash__(self):
        r'''Hashes registration component.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(RegistrationComponent, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def source_pitch_range(self):
        r'''Gets source pitch range of registration component.

        ..  container:: example

            Gets source pitch range of example component:

            ::

                >>> component.source_pitch_range
                PitchRange(range_string='[A0, C8]')

        Returns pitch range or none.
        '''
        return self._source_pitch_range

    @property
    def target_octave_start_pitch(self):
        r'''Gets target octave start pitch of registration component.

        ..  container:: example

            Gets target octave start pitch of example component:

            ::

                >>> component.target_octave_start_pitch
                NumberedPitch(15)

        Returns numbered pitch or none.
        '''
        return self._target_octave_start_pitch



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Slur___init___01():
    r'''Initialize empty slur spanner.
    '''

    slur = Slur()
    assert isinstance(slur, Slur)


def test_spannertools_Slur___init___02():
    r'''You can span across staves if they and
    all their subcontexts are equally named.
    '''

    container = Container(
        r'''
        \context Staff = "foo" <<
            \context Voice = "first" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "second" {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        \context Staff = "foo" <<
            \context Voice = "first" {
                af'8
                a'8
                bf'8
                b'8
            }
            \context Voice = "second" {
                c''8
                cs''8
                d''8
                ef''8
            }
        >>
        '''
        )

    leaves = container[0][0][:] + container[1][0][:]
    slur = Slur()
    attach(slur, leaves)

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "foo" <<
                \context Voice = "first" {
                    c'8 (
                    cs'8
                    d'8
                    ef'8
                }
                \context Voice = "second" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            >>
            \context Staff = "foo" <<
                \context Voice = "first" {
                    af'8
                    a'8
                    bf'8
                    b'8 )
                }
                \context Voice = "second" {
                    c''8
                    cs''8
                    d''8
                    ef''8
                }
            >>
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_spannertools_Slur___init___03():
    '''You can span over liked named staves
    so long as the voices nested in the staves are named the same.
    '''

    container = Container(
        r'''
        \context Staff = "foo" {
            \context Voice = "bar" {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        \context Staff = "foo" {
            \context Voice = "bar" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    slur = Slur()
    leaves = select(container).by_leaf()
    attach(slur, leaves)

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "foo" {
                \context Voice = "bar" {
                    c'8 (
                    cs'8
                    d'8
                    ef'8
                }
            }
            \context Staff = "foo" {
                \context Voice = "bar" {
                    e'8
                    f'8
                    fs'8
                    g'8 )
                }
            }
        }
        '''
        )

    assert inspect_(container).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo___repr___01():
    r'''Tempo returns nonempty string repr.
    '''

    repr = Tempo((1, 8), 48).__repr__()
    assert isinstance(repr, str) and 0 < len(repr)



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_are_scalar_notes_01():
    r'''Notes with the same pitch name are scalar so long
    as pitch numbers differ.
    '''

    notes = [Note('c', (1, 4)), Note('cs', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert selection.are_scalar_notes()


def test_tonalanalysistools_TonalAnalysisAgent_are_scalar_notes_02():
    r'''Notes with different pitch names are scalar so long
    as they differ by exactly one staff space.
    '''

    notes = [Note('c', (1, 4)), Note('d', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert selection.are_scalar_notes()

    notes = [Note('c', (1, 4)), Note('ds', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert selection.are_scalar_notes()

    notes = [Note('c', (1, 4)), Note('b,', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert selection.are_scalar_notes()

    notes = [Note('c', (1, 4)), Note('bf,', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert selection.are_scalar_notes()


def test_tonalanalysistools_TonalAnalysisAgent_are_scalar_notes_03():
    r'''Notes with the same pitch are not scalar.
    '''

    notes = [Note('c', (1, 4)), Note('c', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert not selection.are_scalar_notes()


def test_tonalanalysistools_TonalAnalysisAgent_are_scalar_notes_04():
    r'''Notes separated by more than 1 staff space are not scalar.
    '''

    notes = [Note('c', (1, 4)), Note('e', (1, 4))]
    selection = tonalanalysistools.select(notes)
    assert not selection.are_scalar_notes()


def test_tonalanalysistools_TonalAnalysisAgent_are_scalar_notes_05():
    r'''Contour changes in note sequence qualifies as nonscalar.
    '''

    notes = scoretools.make_notes([0, 2, 4, 5, 4, 2, 0], [(1, 4)])
    selection = tonalanalysistools.select(notes)
    assert not selection.are_scalar_notes()



# -*- coding: utf-8 -*-
import types


def new(argument, **keywords):
    r'''Makes new `argument` with optional `keywords`.

    ..  container:: example

        Makes markup with new direction:

        ::

            >>> markup = Markup('Andante assai', direction=Up).italic()
            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    ^ \markup {
                        \italic
                            "Andante assai"
                        }
                d'4
                e'4
                f'4
            }

        ::

            >>> markup = new(markup, direction=Down)
            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    _ \markup {
                        \italic
                            "Andante assai"
                        }
                d'4
                e'4
                f'4
            }

    Returns new object with type equal to that of `argument`.
    '''
    from abjad.tools import systemtools
    if argument is None:
        return argument
    agent = systemtools.StorageFormatAgent(argument)
    template_dict = agent.get_template_dict()
    recursive_arguments = {}
    for key, value in keywords.items():
        if '__' in key:
            key, divider, subkey = key.partition('__')
            if key not in recursive_arguments:
                recursive_arguments[key] = []
            pair = (subkey, value)
            recursive_arguments[key].append(pair)
            continue
        if key in template_dict or agent.signature_accepts_kwargs:
            template_dict[key] = value
        elif isinstance(getattr(argument, key, None), types.MethodType):
            method = getattr(argument, key)
            result = method(value)
            if isinstance(result, type(argument)):
                argument = result
                template_dict.update(systemtools.StorageFormatAgent(
                    argument).get_template_dict())
        else:
            message = '{} has no key {!r}'.format(type(argument), key)
            raise KeyError(message)
    for key, pairs in recursive_arguments.items():
        recursed_object = getattr(argument, key)
        if recursed_object is None:
            continue
        recursive_template_dict = dict(pairs)
        recursed_object = new(recursed_object, **recursive_template_dict)
        if key in template_dict:
            template_dict[key] = recursed_object
    positional_values = []
    for name in agent.signature_positional_names:
        if name in template_dict:
            positional_values.append(template_dict.pop(name))
    result = type(argument)(*positional_values, **template_dict)
    for name in getattr(argument, '_private_attributes_to_copy', []):
        value = getattr(argument, name, None)
        setattr(result, name, value)
    return result



# -*- coding: utf-8 -*-
import collections
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class NonreducedRatio(AbjadValueObject):
    '''Nonreduced ratio.

    ..  container:: example

        Nonreduced ratio of two numbers:

        ::

            >>> mathtools.NonreducedRatio((2, 4))
            NonreducedRatio((2, 4))

    ..  container:: example

        Nonreduced ratio of three numbers:

            >>> mathtools.NonreducedRatio((2, 4, 2))
            NonreducedRatio((2, 4, 2))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_numbers',
        )

    ### INITIALIZER ###

    def __init__(self, numbers=(1, 1)):
        if isinstance(numbers, type(self)):
            numbers = numbers.numbers
        numbers = tuple(numbers)
        self._numbers = numbers

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''Is true when ratio contains `argument`. Otherwise false.

        Returns true or false.
        '''
        return argument in self._numbers

    def __eq__(self, argument):
        r'''Is true when `argument` is a nonreduced ratio with numerator and
        denominator equal to those of this nonreduced ratio. Otherwise false.

        Returns true or false.
        '''
        if not isinstance(argument, type(self)):
            return False
        argument = type(self)(argument)
        return self.numbers == argument.numbers

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
                >>> print(format(ratio))
                mathtools.NonreducedRatio((2, 4, 2))

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        ..  container:: example

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
                >>> ratio[1]
                4

        Returns integer or tuple.
        '''
        if isinstance(argument, slice):
            return tuple(self._numbers.__getitem__(argument))
        return self._numbers.__getitem__(argument)

    def __hash__(self):
        r'''Hashes non-reduced ratio.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NonreducedRatio, self).__hash__()

    def __iter__(self):
        r'''Iterates ratio.

        Returns generator.
        '''
        return iter(self._numbers)

    def __len__(self):
        r'''Gets length of ratio.

        ..  container:: example

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
                >>> len(ratio)
                3

        Returns integer.
        '''
        return len(self._numbers)

    def __reversed__(self):
        r'''Iterates ratio in reverse.

        Returns generator.
        '''
        return reversed(self._numbers)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=[self.numbers],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    ### PUBLIC METHODS ###

    def count(self, argument):
        r'''Gets count of `argument` in ratio.

        Returns integer.
        '''
        return self._numbers.count(argument)

    def index(self, argument):
        r'''Gets index of `argument` in ratio.

        Returns integer.
        '''
        return self._numbers.index(argument)

    ### PRIVATE PROPERTIES ###

    @property
    def _number_coercer(self):
        return int

    ### PUBLIC PROPERTIES ###

    @property
    def multipliers(self):
        r'''Gets multipliers of nonreduced ratio.

        ..  container:: example

            Nonreduced ratio of two numbers:

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4))
                >>> ratio.multipliers
                (Multiplier(1, 3), Multiplier(2, 3))

        ..  container:: example

            Nonreduced ratio of three numbers:

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
                >>> ratio.multipliers
                (Multiplier(1, 4), Multiplier(1, 2), Multiplier(1, 4))

        Returns tuple of multipliers.
        '''
        from abjad.tools import durationtools
        weight = sum(self.numbers)
        multipliers = [
            durationtools.Multiplier((_, weight))
            for _ in self.numbers
            ]
        multipliers = tuple(multipliers)
        return multipliers

    @property
    def numbers(self):
        r'''Gets numbers of nonreduced ratio.

        ..  container:: example

            Nonreduced ratio of two numbers:

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4))
                >>> ratio.numbers
                (2, 4)

        ..  container:: example

            Nonreduced ratio of three numbers:

            ::

                >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
                >>> ratio.numbers
                (2, 4, 2)

        Set to tuple of two or more numbers.

        Returns tuple of two or more numbers.
        '''
        return self._numbers


collections.Sequence.register(NonreducedRatio)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Tempo___sub___01():

    tempo_1 = Tempo(Duration(1, 4), 60)
    tempo_2 = Tempo(Duration(1, 4), 90)

    result = tempo_1 - tempo_2
    assert result == Tempo(Duration(1, 4), -30)

    result = tempo_2 - tempo_1
    assert result == Tempo(Duration(1, 4), 30)


def test_indicatortools_Tempo___sub___02():

    tempo_1 = Tempo(Duration(1, 8), 42)
    tempo_2 = Tempo(Duration(1, 4), 90)

    result = tempo_1 - tempo_2
    assert result == Tempo(Duration(1, 4), -6)

    result = tempo_2 - tempo_1
    assert result == Tempo(Duration(1, 4), 6)


def test_indicatortools_Tempo___sub___03():

    tempo_1 = Tempo(textual_indication='Langsam')
    tempo_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_1 - tempo_2"
    pytest.raises(ImpreciseTempoError, statement)


def test_indicatortools_Tempo___sub___04():

    tempo_1 = Tempo(Duration(1, 8), (90, 92))
    tempo_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_1 - tempo_2"
    pytest.raises(ImpreciseTempoError, statement)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_LilyPondCommand_format_slot_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    command = indicatortools.LilyPondCommand('break', 'closing')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            \break
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_multiplied_quarter_notes_01():

    multipliers = [(1, 4), (1, 5), (1, 6), (1, 7)]
    notes = scoretools.make_multiplied_quarter_notes([0], multipliers)
    staff = Staff(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 * 1
            c'4 * 4/5
            c'4 * 2/3
            c'4 * 4/7
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie__fuse_leaves_by_immediate_parent_01():
    r'''Fuse leaves in logical tie with same immediate parent.
    '''

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    leaves = select(staff).by_leaf()
    tie = spannertools.Tie()
    attach(tie, leaves)

    logical_tie = inspect_(leaves[1]).get_logical_tie()
    result = logical_tie._fuse_leaves_by_immediate_parent()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'4 ~
            }
            {
                c'4
            }
        }
        '''
        )

    assert len(result) == 2
    assert inspect_(staff).is_well_formed()


def test_selectiontools_LogicalTie__fuse_leaves_by_immediate_parent_02():
    r'''Fuse leaves in logical tie with same immediate parent.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    tie = spannertools.Tie()
    attach(tie, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ~
            c'8 ~
            c'8 ~
            c'8
        }
        '''
        )

    logical_tie = inspect_(staff[1]).get_logical_tie()
    result = logical_tie._fuse_leaves_by_immediate_parent()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'2
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert len(result) == 1


def test_selectiontools_LogicalTie__fuse_leaves_by_immediate_parent_03():
    r'''Fuse leaves in logical tie with same immediate parent.
    '''

    note = Note("c'4")
    logical_tie = inspect_(note).get_logical_tie()
    result = logical_tie._fuse_leaves_by_immediate_parent()
    assert len(result) == 1
    assert inspect_(note).is_well_formed()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


@pytest.mark.skip()
def test_scoretools_fuse_measures_01():
    r'''Fuse unicorporated measures carrying
    time signatures with power-of-two denominators.
    '''

    measure_1 = Measure((1, 8), "c'16 d'16")
    beam = Beam()
    attach(beam, measure_1[:])
    measure_2 = Measure((2, 16), "c'16 d'16")
    slur = Slur()
    attach(slur, measure_2[:])
    staff = Staff([measure_1, measure_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'16 [
                d'16 ]
            }
            {
                \time 2/16
                c'16 (
                d'16 )
            }
        }
        '''
        )

    new = scoretools.fuse_measures(staff[:])

    assert format(new) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'16 [
            d'16 ]
            c'16 (
            d'16 )
        }
        '''
        )

    assert new is not measure_1 and new is not measure_2
    assert len(measure_1) == 0
    assert len(measure_2) == 0
    assert inspect_(new).is_well_formed()


@pytest.mark.skip()
def test_scoretools_fuse_measures_02():
    r'''Fuse measures carrying time signatures with differing
    power-of-two denominators. Helpers selects minimum of two denominators.
    Beams are OK because they attach to leaves rather than containers.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 2/16 e'16 f'16 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16
            }
            {
                \time 2/16
                e'16
                f'16 ]
            }
        }
        '''
        )

    scoretools.fuse_measures(voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'16 [
                d'16
                e'16
                f'16 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


@pytest.mark.skip()
def test_scoretools_fuse_measures_03():
    r'''Fuse measures with differing power-of-two denominators.
    Helpers selects minimum of two denominators.
    Beam attaches to container rather than leaves.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 2/16 e'16 f'16 |")
    beam = Beam()
    attach(beam, voice[0])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16 ]
            }
            {
                \time 2/16
                e'16
                f'16
            }
        }
        '''
        )

    scoretools.fuse_measures(voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'16
                d'16
                e'16
                f'16
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


@pytest.mark.skip()
def test_scoretools_fuse_measures_04():
    r'''Fuse measures with power-of-two-denominators together with measures
    without power-of-two denominators.
    Helpers selects least common multiple of denominators.
    Beams are OK because they attach to leaves rather than containers.
    '''

    measure_1 = Measure((1, 8), "c'8")
    measure_2 = Measure((1, 12), "d'8")
    voice = Voice([measure_1, measure_2])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'8 [
            }
            {
                \time 1/12
                \scaleDurations #'(2 . 3) {
                    d'8 ]
                }
            }
        }
        '''
        )

    scoretools.fuse_measures(voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 5/24
                \scaleDurations #'(2 . 3) {
                    c'8. [
                    d'8 ]
                }
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


@pytest.mark.skip()
def test_scoretools_fuse_measures_05():
    r'''Fusing empty selection returns none.
    '''

    staff = Staff()
    result = scoretools.fuse_measures(staff[:])
    assert result is None


@pytest.mark.skip()
def test_scoretools_fuse_measures_06():
    r'''Fusing selection of only one measure returns measure unaltered.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8")
    staff = Staff([measure])
    new = scoretools.fuse_measures(staff[:])

    assert new is measure


@pytest.mark.skip()
def test_scoretools_fuse_measures_07():
    r'''Fuse three measures.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 1/8 e'16 f'16 || 1/8 g'16 a'16 |")
    leaves = select(voice).by_leaf()
    beam = beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16
            }
            {
                e'16
                f'16
            }
            {
                g'16
                a'16 ]
            }
        }
        '''
        )

    scoretools.fuse_measures(voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/8
                c'16 [
                d'16
                e'16
                f'16
                g'16
                a'16 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


@pytest.mark.skip()
def test_scoretools_fuse_measures_08():
    r'''Measure fusion across intervening container boundaries is undefined.
    '''

    container_1 = Container("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    container_2 = Container("abj: | 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    voice = Voice([container_1, container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    \time 2/8
                    c'8
                    d'8
                }
                {
                    \time 2/8
                    e'8
                    f'8
                }
            }
            {
                {
                    \time 2/8
                    g'8
                    a'8
                }
                {
                    \time 2/8
                    b'8
                    c''8
                }
            }
        }
        '''
        )

    assert pytest.raises(
        AssertionError,
        'scoretools.fuse_measures([voice[0][1], voice[1][0]])',
        )


@pytest.mark.skip()
def test_scoretools_fuse_measures_09():
    r'''Fusing measures with power-of-two denominators
    to measures without power-of-two denominators.
    With change in number of note-heads because of non-power-of-two multiplier.
    '''

    staff = Staff()
    staff.append(Measure((9, 80), "c'64 c' c' c' c' c' c' c' c'"))
    staff.append(Measure((2, 16), "c'16 c'"))

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 9/80
                \scaleDurations #'(4 . 5) {
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                }
            }
            {
                \time 2/16
                c'16
                c'16
            }
        }
        '''
        )

    new = scoretools.fuse_measures(staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 19/80
                \scaleDurations #'(4 . 5) {
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'16 ~
                    c'64
                    c'16 ~
                    c'64
                }
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie__all_leaves_are_in_same_parent_01():

    staff = scoretools.make_repeated_notes(4)
    tie = spannertools.Tie()
    attach(tie, staff[:])

    assert inspect_(staff[0]).get_logical_tie()._all_leaves_are_in_same_parent


def test_selectiontools_LogicalTie__all_leaves_are_in_same_parent_02():

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    leaves = select(staff).by_leaf()
    tie = spannertools.Tie()
    attach(tie, leaves[1:3])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                c'8 ~
            }
            {
                c'8
                c'8
            }
        }
        '''
        )

    assert inspect_(leaves[0]).get_logical_tie()._all_leaves_are_in_same_parent
    assert not inspect_(leaves[1]).get_logical_tie()._all_leaves_are_in_same_parent
    assert not inspect_(leaves[2]).get_logical_tie()._all_leaves_are_in_same_parent
    assert inspect_(leaves[3]).get_logical_tie()._all_leaves_are_in_same_parent


def test_selectiontools_LogicalTie__all_leaves_are_in_same_parent_03():

    staff = Staff(r"\times 2/3 { c'8 c'8 c'8 ~ } \times 2/3 { c'8 c'8 c'8 }")
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \times 2/3 {
                c'8
                c'8
                c'8 ~
            }
            \times 2/3 {
                c'8
                c'8
                c'8
            }
        }
        '''
        )

    assert inspect_(leaves[0]).get_logical_tie()._all_leaves_are_in_same_parent
    assert inspect_(leaves[1]).get_logical_tie()._all_leaves_are_in_same_parent
    assert not inspect_(leaves[2]).get_logical_tie()._all_leaves_are_in_same_parent
    assert not inspect_(leaves[3]).get_logical_tie()._all_leaves_are_in_same_parent
    assert inspect_(leaves[4]).get_logical_tie()._all_leaves_are_in_same_parent
    assert inspect_(leaves[5]).get_logical_tie()._all_leaves_are_in_same_parent



# -*- coding: utf-8 -*-
import copy
import functools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadObject import AbjadObject


@functools.total_ordering
class NoteHead(AbjadObject):
    r'''A note-head.

    ::

        >>> note_head = scoretools.NoteHead(13)
        >>> note_head
        NoteHead("cs''")

    Note heads are immutable.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Note heads'

    __slots__ = (
        '_client',
        '_is_cautionary',
        '_is_forced',
        '_is_parenthesized',
        '_tweak',
        '_written_pitch',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        written_pitch=None,
        client=None,
        is_cautionary=None,
        is_forced=None,
        is_parenthesized=None,
        tweak_pairs=(),
        ):
        from abjad.tools import scoretools
        assert isinstance(client, (type(None), scoretools.Leaf))
        self._client = client
        self._tweak = None
        if isinstance(written_pitch, type(self)):
            note_head = written_pitch
            written_pitch = note_head.written_pitch
            is_cautionary = note_head.is_cautionary
            is_forced = note_head.is_forced
            tweak_pairs = note_head.tweak._get_attribute_pairs()
        elif written_pitch is None:
            written_pitch = 0
        self.written_pitch = written_pitch
        self.is_cautionary = is_cautionary
        self.is_forced = is_forced
        self.is_parenthesized = is_parenthesized
        for tweak_pair in tweak_pairs:
            key, value = tweak_pair
            setattr(self.tweak, key, copy.copy(value))

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        r'''Copies note-head.

        ::

            >>> import copy
            >>> copy.copy(note_head)
            NoteHead("cs''")

        Returns new note-head.
        '''
        return type(self)(*self.__getnewargs__())

    def __eq__(self, argument):
        r'''Is true when `argument` is a note-head with written pitch equal to
        that of this note-head. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.written_pitch == argument.written_pitch
        return self.written_pitch == argument

    def __format__(self, format_specification=''):
        r'''Formats note-head.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        arguments = (
            self.written_pitch,
            None,
            self.is_cautionary,
            self.is_forced,
            self.is_parenthesized,
            self.tweak._get_attribute_pairs(),
            )
        return arguments

    def __hash__(self):
        r'''Hashes note-head.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NoteHead, self).__hash__()

    def __lt__(self, argument):
        r'''Is true when `argument` is a note-head with written pitch greater than
        that of this note-head. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.written_pitch < argument.written_pitch
        try:
            argument = type(self)(argument)
        except (ValueError, TypeError):
            return False
        return self.written_pitch < argument.written_pitch

    def __repr__(self):
        r'''Gets interpreter representation of note-head.

        ::

            >>> note_head
            NoteHead("cs''")

        Returns string.
        '''
        return super(NoteHead, self).__repr__()

    def __str__(self):
        r'''String representation of note-head.

        ::

            >>> str(note_head)
            "cs''"

        Returns string.
        '''
        return self._format_string

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        result = ''
        if self.written_pitch:
            result = str(self.written_pitch)
            if self.is_forced:
                result += '!'
            if self.is_cautionary:
                result += '?'
        return result

    @property
    def _keyword_argument_names(self):
        from abjad.tools import systemtools
        agent = systemtools.StorageFormatAgent(self)
        keyword_argument_names = list(agent.signature_keyword_names)
        if 'client' in keyword_argument_names:
            keyword_argument_names.remove('client')
        if 'tweak_pairs' in keyword_argument_names:
            keyword_argument_names.remove('tweak_pairs')
        keyword_argument_names = tuple(keyword_argument_names)
        return keyword_argument_names

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        arguments = [repr(self._format_string)]
        arguments.extend(self.tweak._get_attribute_pairs())
        arguments = ', '.join([str(x) for x in arguments])
        repr_text = '{}({})'.format(type(self).__name__, arguments)
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if 'client' in names:
            names.remove('client')
        if 'tweak_pairs' in names:
            names.remove('tweak_pairs')
        return systemtools.FormatSpecification(
            self,
            repr_text=repr_text,
            storage_format_kwargs_names=names,
            )

    def _get_lilypond_format(self):
        from abjad.tools import systemtools
        from abjad.tools import scoretools
        # make sure note-head has pitch
        assert self.written_pitch
        result = []
        # format chord note-head with optional tweaks
        if self.is_parenthesized:
            result.append(r'\parenthesize')
        if isinstance(self._client, scoretools.Chord):
            for key, value in vars(self.tweak).items():
                if not key.startswith('_'):
                    result.append(
                        r'\tweak %s %s' % (
                        systemtools.LilyPondFormatManager.format_lilypond_attribute(key),
                        systemtools.LilyPondFormatManager.format_lilypond_value(value)),
                        )
        # format note-head pitch
        kernel = format(self.written_pitch)
        if self.is_forced:
            kernel += '!'
        if self.is_cautionary:
            kernel += '?'
        result.append(kernel)
        result = '\n'.join(result)
        # return formatted note-head
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''Client of note-head.

        ::

            >>> note_head.client is None
            True

        Returns note, chord or none.
        '''
        return self._client

    @property
    def is_cautionary(self):
        r'''Gets and sets cautionary accidental flag.

        Gets cautionary accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_cautionary is None
            True

        Sets cautionary accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_cautionary = True

        Returns true or false.
        '''
        return self._is_cautionary

    @is_cautionary.setter
    def is_cautionary(self, arg):
        if arg is not None:
            arg = bool(arg)
        self._is_cautionary = arg

    @property
    def is_forced(self):
        r'''Gets and sets forced accidental flag.

        Gets forced accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_forced is None
            True

        Sets forced accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_forced = True

        Returns true or false.
        '''
        return self._is_forced

    @is_forced.setter
    def is_forced(self, arg):
        if arg is not None:
            arg = bool(arg)
        self._is_forced = arg

    @property
    def is_parenthesized(self):
        r'''Gets and sets forced accidental flag.

        Gets forced accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_parenthesized is None
            True

        Sets forced accidental flag:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.is_parenthesized = True

        Returns true or false.
        '''
        return self._is_parenthesized

    @is_parenthesized.setter
    def is_parenthesized(self, arg):
        if arg is not None:
            arg = bool(arg)
        self._is_parenthesized = arg

    @property
    def named_pitch(self):
        r'''Named pitch of note-head.

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.named_pitch
            NamedPitch("cs''")

        Returns named pitch.
        '''
        return self.written_pitch

    @property
    def tweak(self):
        r'''LilyPond tweak reservoir of note-head.

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.tweak
            LilyPondNameManager()

        Returns LilyPond tweak reservoir.
        '''
        from abjad.tools import lilypondnametools
        if self._tweak is None:
            self._tweak = lilypondnametools.LilyPondNameManager()
        return self._tweak

    @property
    def written_pitch(self):
        r'''Gets and sets written pitch of note-head.

        Gets written pitch of note-head:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.written_pitch
            NamedPitch("cs''")

        Sets written pitch of note-head:

        ::

            >>> note_head = scoretools.NoteHead("cs''")
            >>> note_head.written_pitch = "d''"
            >>> note_head.written_pitch
            NamedPitch("d''")

        Returns named pitch.
        '''
        return self._written_pitch

    @written_pitch.setter
    def written_pitch(self, arg):
        from abjad.tools import pitchtools
        written_pitch = pitchtools.NamedPitch(arg)
        self._written_pitch = written_pitch



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer___eq___01():

    tree_container_1 = datastructuretools.TreeContainer([])
    tree_container_2 = datastructuretools.TreeContainer([])

    assert format(tree_container_1) == format(tree_container_2)
    assert tree_container_1 != tree_container_2


def test_datastructuretools_TreeContainer___eq___02():

    tree_container_1 = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])
    tree_container_2 = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])

    assert format(tree_container_1) == format(tree_container_2)
    assert tree_container_1 != tree_container_2


def test_datastructuretools_TreeContainer___eq___03():

    tree_container_1 = datastructuretools.TreeContainer([])
    tree_container_2 = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])
    tree_container_3 = datastructuretools.TreeContainer([
        datastructuretools.TreeNode(),
        datastructuretools.TreeNode()
        ])

    assert format(tree_container_1) != format(tree_container_2)
    assert tree_container_1 != tree_container_2
    assert tree_container_1 != tree_container_3
    assert tree_container_2 != tree_container_3



# -*- coding: utf-8 -*-
from __future__ import print_function
from abjad.tools.abctools import AbjadObject


class QuantizationJob(AbjadObject):
    r'''A copiable, picklable class for generating all ``QGrids`` which
    are valid under a given ``SearchTree`` for a sequence
    of ``QEventProxies``:

    ::

        >>> q_event_a = quantizationtools.PitchedQEvent(250, [0, 1])
        >>> q_event_b = quantizationtools.SilentQEvent(500)
        >>> q_event_c = quantizationtools.PitchedQEvent(750, [3, 7])
        >>> proxy_a = quantizationtools.QEventProxy(q_event_a, 0.25)
        >>> proxy_b = quantizationtools.QEventProxy(q_event_b, 0.5)
        >>> proxy_c = quantizationtools.QEventProxy(q_event_c, 0.75)

    ::

        >>> definition = {2: {2: None}, 3: None, 5: None}
        >>> search_tree = quantizationtools.UnweightedSearchTree(definition)

    ::

        >>> job = quantizationtools.QuantizationJob(
        ...     1, search_tree, [proxy_a, proxy_b, proxy_c])

    ``QuantizationJob`` generates ``QGrids`` when called, and stores those
    ``QGrids`` on its ``q_grids`` attribute, allowing them to be recalled
    later, even if pickled:

    ::

        >>> job()
        >>> for q_grid in job.q_grids:
        ...     print(q_grid.rtm_format)
        1
        (1 (1 1 1 1 1))
        (1 (1 1 1))
        (1 (1 1))
        (1 ((1 (1 1)) (1 (1 1))))

    ``QuantizationJob`` is intended to be useful in
    multiprocessing-enabled environments.

    Return ``QuantizationJob`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_job_id',
        '_q_event_proxies',
        '_q_grids',
        '_search_tree',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        job_id=1,
        search_tree=None,
        q_event_proxies=None,
        q_grids=None,
        ):
        from abjad.tools import quantizationtools

        search_tree = search_tree or \
            quantizationtools.UnweightedSearchTree()

        q_event_proxies = q_event_proxies or []

        assert isinstance(search_tree, quantizationtools.SearchTree)
        assert all(
            isinstance(x, quantizationtools.QEventProxy)
            for x in q_event_proxies
            )
        self._job_id = job_id
        self._search_tree = search_tree
        self._q_event_proxies = tuple(q_event_proxies)
        if q_grids is None:
            self._q_grids = ()
        else:
            assert all(
                isinstance(x, quantizationtools.QGrid)
                for x in q_grids
                )
            self._q_grids = tuple(q_grids)

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls quantization job.

        Returns none.
        '''
        from abjad.tools import quantizationtools
        #print('XXX')
        #print(format(self.q_event_proxies[0]))

        q_grid = quantizationtools.QGrid()
        q_grid.fit_q_events(self.q_event_proxies)

        #print(format(q_grid))

        old_q_grids = []
        new_q_grids = [q_grid]

        while new_q_grids:
            q_grid = new_q_grids.pop()
            search_results = self.search_tree(q_grid)
            #print q_grid.rtm_format
            #for x in search_results:
            #    print '\t', x.rtm_format
            new_q_grids.extend(search_results)
            old_q_grids.append(q_grid)

        #for q_grid in old_q_grids:
        #    print('\t', q_grid)
        #print()

        self._q_grids = tuple(old_q_grids)

    def __eq__(self, argument):
        r'''Is true when `argument` is a quantization job with job ID, search tree,
        q-event proxies and q-grids equal to those of this quantization job.
        Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument):
            if self.job_id == argument.job_id:
                if self.search_tree == argument.search_tree:
                    if self.q_event_proxies == argument.q_event_proxies:
                        if self.q_grids == argument.q_grids:
                            return True
        return False

    def __hash__(self):
        r'''Hashes quantization job.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(QuantizationJob, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def job_id(self):
        r'''The job id of the ``QuantizationJob``.

        ::

            >>> job.job_id
            1

        Only meaningful when the job is processed via multiprocessing,
        as the job id is necessary to reconstruct the order of jobs.

        Returns int.
        '''
        return self._job_id

    @property
    def q_event_proxies(self):
        r'''The ``QEventProxies`` the ``QuantizationJob`` was instantiated
        with.

        ::

            >>> for q_event_proxy in job.q_event_proxies:
            ...     print(format(q_event_proxy, 'storage'))
            ... 
            quantizationtools.QEventProxy(
                quantizationtools.PitchedQEvent(
                    offset=durationtools.Offset(250, 1),
                    pitches=(
                        pitchtools.NamedPitch("c'"),
                        pitchtools.NamedPitch("cs'"),
                        ),
                    ),
                durationtools.Offset(1, 4)
                )
            quantizationtools.QEventProxy(
                quantizationtools.SilentQEvent(
                    offset=durationtools.Offset(500, 1),
                    ),
                durationtools.Offset(1, 2)
                )
            quantizationtools.QEventProxy(
                quantizationtools.PitchedQEvent(
                    offset=durationtools.Offset(750, 1),
                    pitches=(
                        pitchtools.NamedPitch("ef'"),
                        pitchtools.NamedPitch("g'"),
                        ),
                    ),
                durationtools.Offset(3, 4)
                )

        Returns tuple.
        '''
        return self._q_event_proxies

    @property
    def q_grids(self):
        r'''The generated ``QGrids``.

        ::

            >>> for q_grid in job.q_grids:
            ...     print(q_grid.rtm_format)
            1
            (1 (1 1 1 1 1))
            (1 (1 1 1))
            (1 (1 1))
            (1 ((1 (1 1)) (1 (1 1))))

        Returns tuple.
        '''
        return self._q_grids

    @property
    def search_tree(self):
        r'''The search tree the ``QuantizationJob`` was instantiated with.

        ::

            >>> job.search_tree
            UnweightedSearchTree(definition={2: {2: None}, 3: None, 5: None})

        Return ``SearchTree`` instance.
        '''
        return self._search_tree



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitchClass___init___01():
    r'''Pitch-class initialization works with numbers.
    '''

    assert isinstance(pitchtools.NumberedPitchClass(0),
        pitchtools.NumberedPitchClass)
    assert isinstance(pitchtools.NumberedPitchClass(0.5),
        pitchtools.NumberedPitchClass)
    assert isinstance(pitchtools.NumberedPitchClass(1),
        pitchtools.NumberedPitchClass)
    assert isinstance(pitchtools.NumberedPitchClass(1.5),
        pitchtools.NumberedPitchClass)
    assert isinstance(pitchtools.NumberedPitchClass(13),
        pitchtools.NumberedPitchClass)
    assert isinstance(pitchtools.NumberedPitchClass(13.5),
        pitchtools.NumberedPitchClass)


def test_pitchtools_NumberedPitchClass___init___02():
    r'''Pitch class initialization works with other pitch-classes.
    '''

    pitch_class = pitchtools.NumberedPitchClass(pitchtools.NumberedPitchClass(0))
    assert isinstance(pitch_class, pitchtools.NumberedPitchClass)

    pitch_class = pitchtools.NumberedPitchClass(pitchtools.NumberedPitchClass(12))
    assert isinstance(pitch_class, pitchtools.NumberedPitchClass)


def test_pitchtools_NumberedPitchClass___init___03():
    r'''Pitch-class initialization works with pitches.
    '''

    pitch_class = pitchtools.NumberedPitchClass(NamedPitch(0))
    assert isinstance(pitch_class, pitchtools.NumberedPitchClass)

    pitch_class = pitchtools.NumberedPitchClass(NamedPitch(12))
    assert isinstance(pitch_class, pitchtools.NumberedPitchClass)


def test_pitchtools_NumberedPitchClass___init___04():
    r'''Pitch class initialization works with notes.
    '''

    note = Note(13, (1, 4))
    pitch_class = pitchtools.NumberedPitchClass(note)
    assert pitch_class == pitchtools.NumberedPitchClass(1)


def test_pitchtools_NumberedPitchClass___init___05():
    r'''Pitch class initialization works with one-note chords.
    '''

    chord = Chord([13], (1, 4))
    pitch_class = pitchtools.NumberedPitchClass(chord)
    assert pitch_class == pitchtools.NumberedPitchClass(1)


def test_pitchtools_NumberedPitchClass___init___06():
    r'''Initialize with named pitch-class instance.
    '''

    npc = pitchtools.NamedPitchClass('cs')
    pitch_class = pitchtools.NumberedPitchClass(npc)
    assert pitch_class == pitchtools.NumberedPitchClass(1)


def test_pitchtools_NumberedPitchClass___init___07():
    r'''Pitch-class initialization raises exception.
    '''

    assert pytest.raises(Exception, "pitchtools.NumberedPitchClass('foo')")


def test_pitchtools_NumberedPitchClass___init___08():
    r'''Pitch-class initialization raises TypeError on rest.
    '''

    rest = Rest((1, 4))
    assert pytest.raises(Exception, 'pitchtools.NumberedPitchClass(rest)')


def test_pitchtools_NumberedPitchClass___init___09():
    r'''Pitch-class initialization raises exception on empty chord.
    '''

    chord = Chord([], (1, 4))
    assert pytest.raises(Exception, 'pitchtools.NumberedPitchClass(chord)')


def test_pitchtools_NumberedPitchClass___init___10():
    r'''Initialize from named pitch-class string.
    '''

    assert pitchtools.NumberedPitchClass('c') == 0
    assert pitchtools.NumberedPitchClass('cs') == 1
    assert pitchtools.NumberedPitchClass('cf') == 11
    assert pitchtools.NumberedPitchClass('css') == 2



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer_insert_01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)

    container = rhythmtreetools.RhythmTreeContainer()
    assert container.children == ()

    container.insert(0, leaf_a)
    assert container.children == (leaf_a,)

    container.insert(0, leaf_b)
    assert container.children == (leaf_b, leaf_a)

    container.insert(1, leaf_c)
    assert container.children == (leaf_b, leaf_c, leaf_a)



# -*- coding: utf-8 -*-
from abjad.tools import abjadbooktools
import textwrap
import unittest
from abjad.tools import stringtools


class AbjadDirectiveTests(unittest.TestCase):

    def setUp(self):
        self.handler = abjadbooktools.SphinxDocumentHandler

    def test_1(self):
        source = textwrap.dedent('''
        ..  abjad::

            note = Note("c'4")
            if True:
                note.written_pitch = "ds,"
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block>
                    <literal_block xml:space="preserve">
                        note = Note("c'4")
                        if True:
                            note.written_pitch = "ds,"
            ''')
        self.assertEqual(result, expected)

    def test_2(self):
        source = textwrap.dedent('''
        ..  abjad::
            :allow-exceptions:
            :hide:
            :no-resize:
            :no-trim:
            :strip-prompt:
            :with-thumbnail:

            note = Note("c'4")
            if True:
                note.written_pitch = "ds,"
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block allow-exceptions="True" hide="True" no-resize="True" no-trim="True" strip-prompt="True" with-thumbnail="True">
                    <literal_block xml:space="preserve">
                        note = Note("c'4")
                        if True:
                            note.written_pitch = "ds,"
            ''')
        self.assertEqual(result, expected)

    def test_3(self):
        source = textwrap.dedent('''
        ..  abjad::
            :allow-exceptions:
            :hide:
            :pages: 1-3, 5, 7-7, 10-8,
            :strip-prompt:

            assert True is False
            ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block allow-exceptions="True" hide="True" pages="(1, 2, 3, 5, 7, 10, 9, 8)" strip-prompt="True">
                    <literal_block xml:space="preserve">
                        assert True is False
            ''')
        self.assertEqual(result, expected)

    def test_4(self):
        source = textwrap.dedent('''
        ..  abjad::
            :stylesheet: non-proportional.ly

            show(Note("c'4"))
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block stylesheet="non-proportional.ly">
                    <literal_block xml:space="preserve">
                        show(Note("c'4"))
            ''')
        self.assertEqual(result, expected)

    def test_5(self):
        source = textwrap.dedent('''
        ..  abjad::
            :no-stylesheet:
            :stylesheet: non-proportional.ly

            show(Note("c'4"))
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block no-stylesheet="True">
                    <literal_block xml:space="preserve">
                        show(Note("c'4"))
            ''')
        self.assertEqual(result, expected)

    def test_6(self):
        source = textwrap.dedent('''
        ..  abjad::
            :text-width: 80
            :with-columns: 2

            show(Note("c'4"))
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_input_block text-width="80" with-columns="2">
                    <literal_block xml:space="preserve">
                        show(Note("c'4"))
            ''')
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Leaf__divide_01():
    r'''Divide chord at D4.
    '''

    staff = Staff("<d' ef' e'>4")
    pitch = NamedPitch('D4')
    treble, bass = staff[0]._divide(pitch)
    staff.extend([treble, bass])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <d' ef' e'>4
            <d' ef' e'>4
            r4
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__divide_02():
    r'''Divide chord at Eb4.
    '''

    staff = Staff("<d' ef' e'>4")
    pitch = NamedPitch('Eb4')
    treble, bass = staff[0]._divide(pitch)
    staff.extend([treble, bass])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <d' ef' e'>4
            <ef' e'>4
            d'4
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__divide_03():
    r'''Divide chord at E4.
    '''

    staff = Staff("<d' ef' e'>4")
    pitch = NamedPitch('E4')
    treble, bass = staff[0]._divide(pitch)
    staff.extend([treble, bass])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <d' ef' e'>4
            e'4
            <d' ef'>4
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-


def timespan_2_contains_timespan_1_improperly(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `timespan_2` contains
    `timespan_1` improperly.

    ::

        >>> relation = timespantools.timespan_2_contains_timespan_1_improperly()
        >>> print(format(relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_2.start_offset <= timespan_1.start_offset'),
                    timespantools.Inequality('timespan_1.stop_offset <= timespan_2.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    ..  container:: example

        ::

            >>> staff = Staff(r"c'8. \p \< fs'16 a'4 af'8 \f \> g'8 ~ g'16 f' e' ef' \p")
            >>> timespan_1 = timespantools.Timespan(Offset(1, 4), Offset(3, 8))
            >>> show(staff) # doctest: +SKIP

        ::

            >>> offset_lists = staff[:]._get_offset_lists()
            >>> time_relation = timespantools.timespan_2_contains_timespan_1_improperly(timespan_1=timespan_1)
            >>> start_index, stop_index = time_relation.get_offset_indices(*offset_lists)
            >>> selected_notes = staff[start_index:stop_index]
            >>> selected_notes
            Selection([Note("a'4")])

        ::

            >>> label(selected_notes).color_leaves('red')
            >>> show(staff) # doctest: +SKIP

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'timespan_2.start_offset <= timespan_1.start_offset',
        'timespan_1.stop_offset <= timespan_2.stop_offset',
        ])

    time_relation = timespantools.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Component__move_indicators_01():

    staff = Staff(r'\clef "bass" c \staccato d e f')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "bass"
            c4 -\staccato
            d4
            e4
            f4
        }
        '''
        )

    assert len(inspect_(staff[0]).get_indicators()) == 2
    assert len(inspect_(staff[1]).get_indicators()) == 0
    assert len(inspect_(staff[2]).get_indicators()) == 0
    assert len(inspect_(staff[3]).get_indicators()) == 0

    staff[0]._move_indicators(staff[2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c4
            d4
            \clef "bass"
            e4 -\staccato
            f4
        }
        '''
        )

    assert len(inspect_(staff[0]).get_indicators()) == 0
    assert len(inspect_(staff[1]).get_indicators()) == 0
    assert len(inspect_(staff[2]).get_indicators()) == 2
    assert len(inspect_(staff[3]).get_indicators()) == 0
